import numpy as np 
from scipy import stats

def mean_std_est(expr,labels):
    n_cl = labels.shape[1]  #n_cl is the number of classes
    n_gene = expr.shape[1]
    U = np.mean(expr,axis=0) # the mean expression level of each gene (mean of each column)
# estimate the mean and standard deviation within each class
    Mu = np.zeros([n_gene,n_cl]) 
    Sigma = np.zeros([n_gene,n_cl])
    for i in range(n_cl):
        Mu[:,i] =np.mean(expr[(labels[:,i]==1),],axis=0)
        Sigma[:,i] = np.sum((expr[(labels[:,i]==1),].T-Mu[:,i].reshape([n_gene,1]))*(expr[(labels[:,i]==1),].T-Mu[:,i].reshape([n_gene,1])),axis=1)

    std = np.sqrt(np.sum(Sigma,axis=1)/(expr.shape[0]-n_cl))

    ### fit a line to the data: x-U, y-std
    slope,intercept,r_value,p_value,slope_std_error=stats.linregress(U,std)
    std_h = intercept+slope*U # the predicted standard deviation

    return U,std_h

def augment_noise_scale(expr,labels,Fold=100,alpha=0.2):
    n_cell,n_genes = expr.shape
    U,std_h = mean_std_est(expr,labels)
    Var = np.random.multivariate_normal(mean=np.zeros(expr.shape[1]),cov=alpha*np.diag(np.square(std_h)),size=n_cell)
    expr1 = expr+Var # add noises to the data
    #labels1=np.tile(labels,(Fold+1,1))

    ### add scaling factors
    U1 = -2#0.5*U_min-0.5*U_max
    U2 = 2#0.5*U_max - 0.5*U_min
    ## (u1,u2) is the range to generate scale factor
    SF = np.zeros(n_cell*Fold)
    SF = np.random.uniform(U1,U2,size=n_cell*Fold)   
    SF = SF.reshape((n_cell*Fold,1))
    SF = np.vstack((np.zeros((expr.shape[0],1)),SF))
    
    return expr1,SF
